"""
A collection of monsters from D&D 5e's SRD

This file was autogenerated by https://github.com/stravajiaxen/5e-srd-to-py
"""

from dungeonsheets.monsters.monsters import Monster
from dungeonsheets.stats import Ability


class OchreJelly(Monster):
    """
    Amorphous.
      The jelly can move through a space as narrow as 1 inch wide without
      squeezing.
    Spider Climb.
      The jelly can climb difficult surfaces, including upside down on
      ceilings, without needing to make an ability check.
    Pseudopod.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 9 (2d6 +
      2) bludgeoning damage plus 3 (1d6) acid damage.
    """
    name = "Ochre Jelly"
    description = "Large ooze, unaligned"
    challenge_rating = 2
    armor_class = 8
    skills = ""
    senses = "Blindsight 60 ft. (blind beyond this radius), Passive Perception 8"
    languages = ""
    strength = Ability(15)
    dexterity = Ability(6)
    constitution = Ability(14)
    intelligence = Ability(2)
    wisdom = Ability(6)
    charisma = Ability(1)
    speed = 10
    swim_speed = 0
    fly_speed = 0
    climb_speed = 10
    hp_max = 45
    hit_dice = "6d10"
    spells = []


class Octopus(Monster):
    """
    Hold Breath.
      While out of water, the octopus can hold its breath for 30 minutes.
    Underwater Camouflage.
      The octopus has advantage on Dexterity (Stealth) checks made while
      underwater.
    Water Breathing.
      The octopus can breathe only underwater.
    Tentacles.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 1
      bludgeoning damage, and the target is grappled (escape DC 10). Until
      this grapple ends, the octopus can't use its tentacles on another
      target.
    Ink Cloud.
      A 5-foot-radius cloud of ink extends all around the octopus if it is
      underwater. The area is heavily obscured for 1 minute, although a
      significant current can disperse the ink. After releasing the ink, the
      octopus can use the Dash action as a bonus action.
    """
    name = "Octopus"
    description = "Small beast, unaligned"
    challenge_rating = 0
    armor_class = 12
    skills = "Perception +2, Stealth +4"
    senses = "Darkvision 30 ft., Passive Perception 12"
    languages = ""
    strength = Ability(4)
    dexterity = Ability(15)
    constitution = Ability(11)
    intelligence = Ability(3)
    wisdom = Ability(10)
    charisma = Ability(4)
    speed = 5
    swim_speed = 30
    fly_speed = 0
    climb_speed = 0
    hp_max = 3
    hit_dice = "1d6"
    spells = []


class Ogre(Monster):
    """
    Greatclub.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 13 (2d8
      + 4) bludgeoning damage.
    Javelin.
      Melee or Ranged Weapon Attack: +6 to hit, reach 5 ft. or range 30/120
      ft., one target. Hit: 11 (2d6 + 4) piercing damage.
    """
    name = "Ogre"
    description = "Large giant, chaotic evil"
    challenge_rating = 2
    armor_class = 11
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 8"
    languages = "Common, Giant"
    strength = Ability(19)
    dexterity = Ability(8)
    constitution = Ability(16)
    intelligence = Ability(5)
    wisdom = Ability(7)
    charisma = Ability(7)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 59
    hit_dice = "7d10"
    spells = []


class OgreZombie(Monster):
    """
    Undead Fortitude.
      If damage reduces the zombie to 0 hit points, it must make a
      Constitution saving throw with a DC of 5+the damage taken, unless the
      damage is radiant or from a critical hit. On a success, the zombie
      drops to 1 hit point instead.
    Morningstar.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 13 (2d8
      + 4) bludgeoning damage.
    """
    name = "Ogre Zombie"
    description = "Large undead, neutral evil"
    challenge_rating = 2
    armor_class = 8
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 8"
    languages = "understands Common and Giant but can't speak"
    strength = Ability(19)
    dexterity = Ability(6)
    constitution = Ability(18)
    intelligence = Ability(3)
    wisdom = Ability(6)
    charisma = Ability(5)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 85
    hit_dice = "9d10"
    spells = []


class Oni(Monster):
    """
    Innate Spellcasting.
      The oni's innate spellcasting ability is Charisma (spell save DC 13).
      The oni can innately cast the following spells, requiring no material
      components:
      
  
      
      At will: darkness, invisibility
      
      1/day each: charm person, cone of cold, gaseous form, sleep
    Magic Weapons.
      The oni's weapon attacks are magical.
    Regeneration.
      The oni regains 10 hit points at the start of its turn if it has at
      least 1 hit point.
    Multiattack.
      The oni makes two attacks, either with its claws or its glaive.
    Claw (Oni Form Only).
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 8 (1d8 +
      4) slashing damage.
    Glaive.
      Melee Weapon Attack: +7 to hit, reach 10 ft., one target. Hit: 15
      (2d10 + 4) slashing damage, or 9 (1d10 + 4) slashing damage in Small
      or Medium form.
    Change Shape.
      The oni magically polymorphs into a Small or Medium humanoid, into a
      Large giant, or back into its true form. Other than its size, its
      statistics are the same in each form. The only equipment that is
      transformed is its glaive, which shrinks so that it can be wielded in
      humanoid form. If the oni dies, it reverts to its true form, and its
      glaive reverts to its normal size.
    """
    name = "Oni"
    description = "Large giant, lawful evil"
    challenge_rating = 7
    armor_class = 16
    skills = "Arcana +5, Deception +8, Perception +4"
    senses = "Darkvision 60 ft., Passive Perception 14"
    languages = "Common, Giant"
    strength = Ability(19)
    dexterity = Ability(11)
    constitution = Ability(16)
    intelligence = Ability(14)
    wisdom = Ability(12)
    charisma = Ability(15)
    speed = 30
    swim_speed = 0
    fly_speed = 30
    climb_speed = 0
    hp_max = 110
    hit_dice = "13d10"
    spells = []


class Orc(Monster):
    """
    Aggressive.
      As a bonus action, the orc can move up to its speed toward a hostile
      creature that it can see.
    Greataxe.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 9 (1d12
      + 3) slashing damage.
    Javelin.
      Melee or Ranged Weapon Attack: +5 to hit, reach 5 ft. or range 30/120
      ft., one target. Hit: 6 (1d6 + 3) piercing damage.
    """
    name = "Orc"
    description = "Medium humanoid, chaotic evil"
    challenge_rating = 0.5
    armor_class = 13
    skills = "Intimidation +2"
    senses = "Darkvision 60 ft., Passive Perception 10"
    languages = "Common, Orc"
    strength = Ability(16)
    dexterity = Ability(12)
    constitution = Ability(16)
    intelligence = Ability(7)
    wisdom = Ability(11)
    charisma = Ability(10)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 15
    hit_dice = "2d8"
    spells = []


class Otyugh(Monster):
    """
    Limited Telepathy.
      The otyugh can magically transmit simple messages and images to any
      creature within 120 ft. of it that can understand a language. This
      form of telepathy doesn't allow the receiving creature to
      telepathically respond.
    Multiattack.
      The otyugh makes three attacks: one with its bite and two with its
      tentacles.
    Bite.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 12 (2d8
      + 3) piercing damage. If the target is a creature, it must succeed on
      a DC 15 Constitution saving throw against disease or become poisoned
      until the disease is cured. Every 24 hours that elapse, the target
      must repeat the saving throw, reducing its hit point maximum by 5
      (1d10) on a failure. The disease is cured on a success. The target
      dies if the disease reduces its hit point maximum to 0. This reduction
      to the target's hit point maximum lasts until the disease is cured.
    Tentacle.
      Melee Weapon Attack: +6 to hit, reach 10 ft., one target. Hit: 7 (1d8
      + 3) bludgeoning damage plus 4 (1d8) piercing damage. If the target is
      Medium or smaller, it is grappled (escape DC 13) and restrained until
      the grapple ends. The otyugh has two tentacles, each of which can
      grapple one target.
    Tentacle Slam.
      The otyugh slams creatures grappled by it into each other or a solid
      surface. Each creature must succeed on a DC 14 Constitution saving
      throw or take 10 (2d6 + 3) bludgeoning damage and be stunned until the
      end of the otyugh's next turn. On a successful save, the target takes
      half the bludgeoning damage and isn't stunned.
    """
    name = "Otyugh"
    description = "Large aberration, neutral"
    challenge_rating = 5
    armor_class = 14
    skills = ""
    senses = "Darkvision 120 ft., Passive Perception 11"
    languages = "Otyugh"
    strength = Ability(16)
    dexterity = Ability(11)
    constitution = Ability(19)
    intelligence = Ability(6)
    wisdom = Ability(13)
    charisma = Ability(6)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 114
    hit_dice = "12d10"
    spells = []


class Owl(Monster):
    """
    Flyby.
      The owl doesn't provoke opportunity attacks when it flies out of an
      enemy's reach.
    Keen Hearing and Sight.
      The owl has advantage on Wisdom (Perception) checks that rely on
      hearing or sight.
    Talons.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 1
      slashing damage.
    """
    name = "Owl"
    description = "Tiny beast, unaligned"
    challenge_rating = 0
    armor_class = 11
    skills = "Perception +3, Stealth +3"
    senses = "Darkvision 120 ft., Passive Perception 13"
    languages = ""
    strength = Ability(3)
    dexterity = Ability(13)
    constitution = Ability(8)
    intelligence = Ability(2)
    wisdom = Ability(12)
    charisma = Ability(7)
    speed = 5
    swim_speed = 0
    fly_speed = 60
    climb_speed = 0
    hp_max = 1
    hit_dice = "1d4"
    spells = []


class Owlbear(Monster):
    """
    Keen Sight and Smell.
      The owlbear has advantage on Wisdom (Perception) checks that rely on
      sight or smell.
    Multiattack.
      The owlbear makes two attacks: one with its beak and one with its
      claws.
    Beak.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one creature. Hit: 10
      (1d10 + 5) piercing damage.
    Claws.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 14 (2d8
      + 5) slashing damage.
    """
    name = "Owlbear"
    description = "Large monstrosity, unaligned"
    challenge_rating = 3
    armor_class = 13
    skills = "Perception +3"
    senses = "Darkvision 60 ft., Passive Perception 13"
    languages = ""
    strength = Ability(20)
    dexterity = Ability(12)
    constitution = Ability(17)
    intelligence = Ability(3)
    wisdom = Ability(12)
    charisma = Ability(7)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 59
    hit_dice = "7d10"
    spells = []



import re

from jinja2 import Environment, PackageLoader


from dungeonsheets.stats import mod_str, ability_mod_str, stat_abbreviation
from dungeonsheets.monsters import challenge_rating_to_xp


# A dice string, with optional backticks: ``1d6 + 3``
dice_re = re.compile(r"`*(\d+d\d+(?:\s*\+\s*\d+)?)`*")


def jinja_environment():
    """Prepare a new environment for Jinja templates.

    This function loads filters that are agnostic to the output
    format. Format specific filters (e.g. html, latex, etc.) should be
    added to ``jinja_env.filters``.

    Returns
    -------
    jinja_env
      The newly created jinja environment.

    """
    jinja_env = Environment(
        loader=PackageLoader("dungeonsheets", "forms"),
        block_start_string="[%",
        block_end_string="%]",
        variable_start_string="[[",
        variable_end_string="]]",
    )
    jinja_env.filters["mod_str"] = mod_str
    jinja_env.filters["ability_mod_str"] = ability_mod_str
    jinja_env.filters["stat_abbreviation"] = stat_abbreviation
    jinja_env.filters["challenge_rating_to_xp"] = challenge_rating_to_xp
    return jinja_env



"""
A collection of monsters from D&D 5e's SRD

This file was autogenerated by https://github.com/stravajiaxen/5e-srd-to-py
"""

from dungeonsheets.monsters.monsters import Monster
from dungeonsheets.stats import Ability


class Lamia(Monster):
    """
    Innate Spellcasting.
      The lamia's innate spellcasting ability is Charisma (spell save DC
      13). It can innately cast the following spells, requiring no material
      components. At will: disguise self (any humanoid form), major image
      3/day each: charm person, mirror image, scrying, suggestion 1/day:
      geas
    Multiattack.
      The lamia makes two attacks: one with its claws and one with its
      dagger or Intoxicating Touch.
    Claws.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 14 (2d10
      + 3) slashing damage.
    Dagger.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 5 (1d4 +
      3) piercing damage.
    Intoxicating Touch.
      Melee Spell Attack: +5 to hit, reach 5 ft., one creature. Hit: The
      target is magically cursed for 1 hour. Until the curse ends, the
      target has disadvantage on Wisdom saving throws and all ability
      checks.
    """
    name = "Lamia"
    description = "Large monstrosity, chaotic evil"
    challenge_rating = 4
    armor_class = 13
    skills = "Deception +7, Insight +4, Stealth +3"
    senses = "Darkvision 60 ft., Passive Perception 12"
    languages = "Abyssal, Common"
    strength = Ability(16)
    dexterity = Ability(13)
    constitution = Ability(15)
    intelligence = Ability(14)
    wisdom = Ability(15)
    charisma = Ability(16)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 97
    hit_dice = "13d10"
    spells = []


class Lemure(Monster):
    """
    Devil's Sight.
      Magical darkness doesn't impede the lemure's darkvision.
    Hellish Rejuvenation.
      A lemure that dies in the Nine Hells comes back to life with all its
      hit points in 1d10 days unless it is killed by a good-aligned creature
      with a bless spell cast on that creature or its remains are sprinkled
      with holy water.
    Fist.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 2 (1d4)
      bludgeoning damage.
    """
    name = "Lemure"
    description = "Medium fiend, lawful evil"
    challenge_rating = 0
    armor_class = 7
    skills = ""
    senses = "Darkvision 120 ft., Passive Perception 10"
    languages = "understands infernal but can't speak"
    strength = Ability(10)
    dexterity = Ability(5)
    constitution = Ability(11)
    intelligence = Ability(1)
    wisdom = Ability(11)
    charisma = Ability(3)
    speed = 15
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 13
    hit_dice = "3d8"
    spells = []


class Lich(Monster):
    """
    Legendary Resistance.
      If the lich fails a saving throw, it can choose to succeed instead.
    Rejuvenation.
      If it has a phylactery, a destroyed lich gains a new body in 1d10
      days, regaining all its hit points and becoming active again. The new
      body appears within 5 feet of the phylactery.
    Spellcasting.
      The lich is an 18th-level spellcaster. Its spellcasting ability is
      Intelligence (spell save DC 20, +12 to hit with spell attacks). The
      lich has the following wizard spells prepared:
      
  
      
      - Cantrips (at will): mage hand, prestidigitation, ray of frost
      
      - 1st level (4 slots): detect magic, magic missile, shield,
      thunderwave
      
      - 2nd level (3 slots): acid arrow, detect thoughts, invisibility,
      mirror image
      
      - 3rd level (3 slots): animate dead, counterspell, dispel magic,
      fireball
      
      - 4th level (3 slots): blight, dimension door
      
      - 5th level (3 slots): cloudkill, scrying
      
      - 6th level (1 slot): disintegrate, globe of invulnerability
      
      - 7th level (1 slot): finger of death, plane shift
      
      - 8th level (1 slot): dominate monster, power word stun
      
      - 9th level (1 slot): power word kill
    Turn Resistance.
      The lich has advantage on saving throws against any effect that turns
      undead.
    Paralyzing Touch.
      Melee Spell Attack: +12 to hit, reach 5 ft., one creature. Hit: 10
      (3d6) cold damage. The target must succeed on a DC 18 Constitution
      saving throw or be paralyzed for 1 minute. The target can repeat the
      saving throw at the end of each of its turns, ending the effect on
      itself on a success.
    """
    name = "Lich"
    description = "Medium undead, any evil alignment"
    challenge_rating = 21
    armor_class = 17
    skills = "Arcana +18, History +12, Insight +9, Perception +9"
    senses = "Truesight 120 ft., Passive Perception 19"
    languages = "Common plus up to five other languages"
    strength = Ability(11)
    dexterity = Ability(16)
    constitution = Ability(16)
    intelligence = Ability(20)
    wisdom = Ability(14)
    charisma = Ability(16)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 135
    hit_dice = "18d8"
    spells = ["mage hand", "prestidigitation", "ray of frost", "detect magic", "magic missile", "shield", "thunderwave", "acid arrow", "detect thoughts", "invisibility", "mirror image", "animate dead", "counterspell", "dispel magic", "fireball", "blight", "dimension door", "cloudkill", "scrying", "disintegrate", "globe of invulnerability", "finger of death", "plane shift", "dominate monster", "power word stun", "power word kill"]


class Lion(Monster):
    """
    Keen Smell.
      The lion has advantage on Wisdom (Perception) checks that rely on
      smell.
    Pack Tactics.
      The lion has advantage on an attack roll against a creature if at
      least one of the lion's allies is within 5 ft. of the creature and the
      ally isn't incapacitated.
    Pounce.
      If the lion moves at least 20 ft. straight toward a creature and then
      hits it with a claw attack on the same turn, that target must succeed
      on a DC 13 Strength saving throw or be knocked prone. If the target is
      prone, the lion can make one bite attack against it as a bonus action.
    Running Leap.
      With a 10-foot running start, the lion can long jump up to 25 ft..
    Bite.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 7 (1d8 +
      3) piercing damage.
    Claw.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 6 (1d6 +
      3) slashing damage.
    """
    name = "Lion"
    description = "Large beast, unaligned"
    challenge_rating = 1
    armor_class = 12
    skills = "Perception +3, Stealth +6"
    senses = "Passive Perception 13"
    languages = ""
    strength = Ability(17)
    dexterity = Ability(15)
    constitution = Ability(13)
    intelligence = Ability(3)
    wisdom = Ability(12)
    charisma = Ability(8)
    speed = 50
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 26
    hit_dice = "4d10"
    spells = []


class Lizard(Monster):
    """
    Bite.
      Melee Weapon Attack: +0 to hit, reach 5 ft., one target. Hit: 1
      piercing damage.
    """
    name = "Lizard"
    description = "Tiny beast, unaligned"
    challenge_rating = 0
    armor_class = 10
    skills = ""
    senses = "Darkvision 30 ft., Passive Perception 9"
    languages = ""
    strength = Ability(2)
    dexterity = Ability(11)
    constitution = Ability(10)
    intelligence = Ability(1)
    wisdom = Ability(8)
    charisma = Ability(3)
    speed = 20
    swim_speed = 0
    fly_speed = 0
    climb_speed = 20
    hp_max = 2
    hit_dice = "1d4"
    spells = []


class Lizardfolk(Monster):
    """
    Hold Breath.
      The lizardfolk can hold its breath for 15 minutes.
    Multiattack.
      The lizardfolk makes two melee attacks, each one with a different
      weapon.
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 5 (1d6 +
      2) piercing damage.
    Heavy Club.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 5 (1d6 +
      2) bludgeoning damage.
    Javelin.
      Melee or Ranged Weapon Attack: +4 to hit, reach 5 ft. or range 30/120
      ft., one target. Hit: 5 (1d6 + 2) piercing damage.
    Spiked Shield.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 5 (1d6 +
      2) piercing damage.
    """
    name = "Lizardfolk"
    description = "Medium humanoid, neutral"
    challenge_rating = 0.5
    armor_class = 15
    skills = "Perception +3, Stealth +4, Survival +5"
    senses = "Passive Perception 13"
    languages = "Draconic"
    strength = Ability(15)
    dexterity = Ability(10)
    constitution = Ability(13)
    intelligence = Ability(7)
    wisdom = Ability(12)
    charisma = Ability(7)
    speed = 30
    swim_speed = 30
    fly_speed = 0
    climb_speed = 0
    hp_max = 22
    hit_dice = "4d8"
    spells = []



#!/usr/bin/env python

"""Program to take character definitions and build a PDF of the
character sheet."""

import logging
import argparse
import os
import subprocess
import warnings
import re
from pathlib import Path
from multiprocessing import Pool, cpu_count
from itertools import product
from typing import Union, Sequence, Optional, List

from dungeonsheets import (
    character as _char,
    exceptions,
    readers,
    latex,
    epub,
    monsters,
    forms,
    random_tables,
)
from dungeonsheets.content_registry import find_content
from dungeonsheets.fill_pdf_template import (
    create_character_pdf_template,
    create_personality_pdf_template,
    create_spells_pdf_template,
)
from dungeonsheets.character import Character
from dungeonsheets.content import Creature


log = logging.getLogger(__name__)

ORDINALS = {
    1: "1st",
    2: "2nd",
    3: "3rd",
    4: "4th",
    5: "5th",
    6: "6th",
    7: "7th",
    8: "8th",
    9: "9th",
}


PDFTK_CMD = "pdftk"


jinja_env = forms.jinja_environment()
jinja_env.filters["rst_to_latex"] = latex.rst_to_latex
jinja_env.filters["rst_to_html"] = epub.rst_to_html
jinja_env.filters["to_heading_id"] = epub.to_heading_id


# Custom types
File = Union[Path, str]


class CharacterRenderer():
    def __init__(self, template_name: str):
        self.template_name = template_name
        
    def __call__(self, character: Character, content_suffix: str = "tex", use_dnd_decorations: bool = False):
        template = jinja_env.get_template(self.template_name.format(suffix=content_suffix))
        return template.render(character=character,
                               use_dnd_decorations=use_dnd_decorations, ordinals=ORDINALS)


create_character_sheet_content = CharacterRenderer("character_sheet_template.{suffix}")
create_subclasses_content = CharacterRenderer("subclasses_template.{suffix}")
create_features_content = CharacterRenderer("features_template.{suffix}")
create_magic_items_content = CharacterRenderer("magic_items_template.{suffix}")
create_spellbook_content = CharacterRenderer("spellbook_template.{suffix}")
create_infusions_content = CharacterRenderer("infusions_template.{suffix}")
create_druid_shapes_content = CharacterRenderer("druid_shapes_template.{suffix}")


def create_monsters_content(
    monsters: Sequence[Union[monsters.Monster, str]],
    suffix: str,
    use_dnd_decorations: bool = False,
) -> str:
    # Convert strings to Monster objects
    template = jinja_env.get_template(f"monsters_template.{suffix}")
    spell_list = [spell for monster in monsters for spell in monster.spells]
    return template.render(monsters=monsters,
                           use_dnd_decorations=use_dnd_decorations, spell_list=spell_list)


def create_party_summary_content(
    party: Sequence[Creature],
    summary_rst: str,
    suffix: str,
    use_dnd_decorations: bool = False,
) -> str:
    log.debug("Preparing summary table for party: %s", party)
    template = jinja_env.get_template(f"party_summary_template.{suffix}")
    return template.render(
        party=party, summary=summary_rst, use_dnd_decorations=use_dnd_decorations
    )


def create_random_tables_content(
    tables: Sequence[random_tables.RandomTable],
    suffix: str,
    use_dnd_decorations: bool = False,
) -> str:
    template = jinja_env.get_template(f"random_tables_template.{suffix}")
    return template.render(
        tables=tables,
        use_dnd_decorations=use_dnd_decorations,
    )


def create_extra_gm_content(sections: Sequence, suffix: str, use_dnd_decorations: bool=False):
    """Create content for arbitrary additional text provided in a GM sheet.

    Parameters
    ==========
    sections
      Subclasses of Content that will each be included as new sections
      in the output document.
    
    """
    template = jinja_env.get_template(f"extra_gm_content.{suffix}")
    return template.render(
        sections=sections, use_dnd_decorations=use_dnd_decorations
    )


def make_sheet(
    sheet_file: File,
    flatten: bool = False,
    output_format: str = "pdf",
    fancy_decorations: bool = False,
    debug: bool = False,
):
    """Make a character or GM sheet into a PDF.
    Parameters
    ----------
    sheet_file
      File (.py) to load character from. Will save PDF using same name
    flatten : bool, optional
      If true, the resulting PDF will look better and won't be
      fillable form.
    output_format
      Either "pdf" or "epub" to generate a PDF file or an EPUB file.
    fancy_decorations : bool, optional
      Use fancy page layout and decorations for extra sheets, namely
      the dnd style file: https://github.com/rpgtex/DND-5e-LaTeX-Template.
    debug : bool, optional
      Provide extra info and preserve temporary files.

    """
    # Parse the file
    sheet_file = Path(sheet_file)
    sheet_props = readers.read_sheet_file(sheet_file)
    # Create the sheet
    if sheet_props.get("sheet_type", "") == "gm":
        ret = make_gm_sheet(
            gm_file=sheet_file,
            output_format=output_format,
            fancy_decorations=fancy_decorations,
            debug=debug,
        )
    else:
        ret = make_character_sheet(
            char_file=sheet_file,
            flatten=flatten,
            output_format=output_format,
            fancy_decorations=fancy_decorations,
            debug=debug,
        )
    return ret


format_suffixes = {
    "pdf": "tex",
    "epub": "html",
}


def make_gm_sheet(
    gm_file: Union[str, Path],
    output_format: str = "pdf",
    fancy_decorations: bool = False,
    debug: bool = False,
):
    """Prepare a PDF character sheet from the given character file.

    Parameters
    ----------
    gm_file
      The file with the gm_sheet definitions.
    output_format
      Either "pdf" or "epub" to generate a PDF file or an EPUB file.
    fancy_decorations
      Use fancy page layout and decorations for extra sheets, namely
      the dnd style file: https://github.com/rpgtex/DND-5e-LaTeX-Template.
    debug
      Provide extra info and preserve temporary files.

    """
    # Parse the GM file and filename
    gm_file = Path(gm_file)
    basename = gm_file.stem
    gm_props = readers.read_sheet_file(gm_file)
    session_title = gm_props.pop("session_title", f"GM Notes: {basename}")
    # Create the intro tex
    content_suffix = format_suffixes[output_format]
    content = [
        jinja_env.get_template(f"preamble.{content_suffix}").render(
            use_dnd_decorations=fancy_decorations,
            title=session_title,
        )
    ]
    # Add the party stats table and session summary
    party = []
    for char_file in gm_props.pop("party", []):
        # Resolve the file path
        char_file = Path(char_file)
        if not char_file.is_absolute():
            char_file = gm_file.parent / char_file
        char_file = char_file.resolve()
        # Load the character file
        log.debug(f"Loading party member: {char_file}")
        character_props = readers.read_sheet_file(char_file)
        member = _char.Character.load(character_props)
        party.append(member)
    summary = gm_props.pop("summary", "")
    content.append(
        create_party_summary_content(
            party,
            summary_rst=summary,
            suffix=content_suffix,
            use_dnd_decorations=fancy_decorations,
        )
    )
    # Parse any extra homebrew sections, etc.
    content.append(
        create_extra_gm_content(sections=gm_props.pop("extra_content", []),
                                suffix=content_suffix,
                                use_dnd_decorations=fancy_decorations)
    )
    # Add the monsters
    monsters_ = []
    for monster in gm_props.pop("monsters", []):
        if isinstance(monster, monsters.Monster):
            # It's already a monster, so just add it
            new_monster = monster
        else:
            try:
                MyMonster = find_content(monster, valid_classes=[monsters.Monster])
            except exceptions.ContentNotFound:
                msg = f"Monster '{monster}' not found. Please add it to ``monsters.py``"
                warnings.warn(msg)
                continue
            else:
                new_monster = MyMonster()
        monsters_.append(new_monster)
    if len(monsters_) > 0:
        content.append(
            create_monsters_content(
                monsters_, suffix=content_suffix, use_dnd_decorations=fancy_decorations
            )
        )
    # Add the random tables
    tables = [
        find_content(s, valid_classes=[random_tables.RandomTable])
        for s in gm_props.pop("random_tables", [])
    ]
    content.append(
        create_random_tables_content(
            tables=tables,
            suffix=content_suffix,
            use_dnd_decorations=fancy_decorations,
        )
    )
    # Add the closing TeX
    content.append(
        jinja_env.get_template(f"postamble.{format_suffixes[output_format]}").render(
            use_dnd_decorations=fancy_decorations
        )
    )
    # Warn about any unhandled sheet properties
    gm_props.pop("dungeonsheets_version")
    gm_props.pop("sheet_type")
    if len(gm_props.keys()) > 0:
        msg = f"Unhandled attributes in '{str(gm_file)}': {','.join(gm_props.keys())}"
        log.warning(msg)
        warnings.warn(msg)
    # Produce the combined output depending on the format requested
    if output_format == "pdf":
        # Typeset combined LaTeX file
        try:
            if len(content) > 2:
                latex.create_latex_pdf(
                    tex="".join(content),
                    basename=basename,
                    keep_temp_files=debug,
                    use_dnd_decorations=fancy_decorations,
                )
        except exceptions.LatexNotFoundError:
            log.warning(f"``pdflatex`` not available. Skipping {basename}")
    elif output_format == "epub":
        chapters = {session_title: "".join(content)}
        # Make sheets in the epub for each party member
        for char in party:
            char_html = make_character_content(char, "html",
                                               fancy_decorations=fancy_decorations)
            chapters[char.name] = "".join(char_html)
        # Create the combined HTML file
        epub.create_epub(
            chapters=chapters,
            basename=basename,
            title=session_title,
            use_dnd_decorations=fancy_decorations,
        )
    else:
        raise exceptions.UnknownOutputFormat(
            f"Unknown output format requested: {output_format}. Valid options are:"
            " 'pdf', 'epub'"
        )


def make_character_content(
        character: Character,
        content_format: str,
        fancy_decorations: bool = False,) -> List[str]:
    """Prepare the inner content for a character sheet.

    This will produce a fully renderable document, suitable for
    passing to routines in either the ``epub`` or ``latex``
    modules. If *content_format* is ``"html"``, the returned content
    is just the portion that would be found inside the
    ``<body></body>`` tag.

    If *content_format* is ``"tex"``, the content returned will not
    include the character, spell list, or biography sheets, since
    these are currently processed through fillable PDFs.

    Parameters
    ----------
    character
      The character to render content for.
    content_format
      Which markup syntax to use, "tex" or "html".
    fancy_decorations
      Use fancy page layout and decorations for extra sheets, namely
      the dnd style file for *tex*, or extended CSS for *html*.
    
    Returns
    -------
    content
      The list of rendered character sheet contents for *character* in
      markup format *content_format*.

    """
    # Preamble, empty for HTML
    content = [
        jinja_env.get_template(f"preamble.{content_format}").render(
            use_dnd_decorations=fancy_decorations,
            title="Features, Magical Items and Spells",
        )
    ]
    # Make the character sheet, and background pages if producing HTML
    if content_format != "tex":
        content.append(create_character_sheet_content(character,
                                                      content_suffix=content_format,
                                                      use_dnd_decorations=fancy_decorations))
    # Create a list of subcasses, features, spells, etc
    if character.subclasses:
        content.append(create_subclasses_content(character,
                                                 content_suffix=content_format,
                                                 use_dnd_decorations=fancy_decorations)
                       )
    if character.features:
        content.append(
            create_features_content(character, content_suffix=content_format, use_dnd_decorations=fancy_decorations)
        )
    if character.magic_items:
        content.append(
            create_magic_items_content(character, content_suffix=content_format, use_dnd_decorations=fancy_decorations)
        )
    if character.is_spellcaster:
        content.append(
            create_spellbook_content(character, content_suffix=content_format, use_dnd_decorations=fancy_decorations)
        )
    if len(getattr(character, "infusions", [])) > 0:
        content.append(
            create_infusions_content(character, content_suffix=content_format, use_dnd_decorations=fancy_decorations)
        )

    # Create a list of Druid wild_shapes
    if len(getattr(character, "all_wild_shapes", [])) > 0:
        content.append(
            create_druid_shapes_content(character, content_suffix=content_format, use_dnd_decorations=fancy_decorations)
        )
    # Postamble, empty for HTML
    content.append(
        jinja_env.get_template(f"postamble.{content_format}").render(
            use_dnd_decorations=fancy_decorations
        )
    )        
    return content


def make_character_sheet(
    char_file: Union[str, Path],
    character: Optional[Character] = None,
    flatten: bool = False,
    output_format: str = "pdf",
    fancy_decorations: bool = False,
    debug: bool = False,
):
    """Prepare a PDF character sheet from the given character file.

    Parameters
    ----------
    basename
      The basename for saving files (PDFs, etc).
    character
      If provided, will not load from the character file, just use
      file for PDF name
    flatten
      If true, the resulting PDF will look better and won't be
      fillable form.
    output_format
      Either "pdf" or "epub" to generate a PDF file or an EPUB file.
    fancy_decorations
      Use fancy page layout and decorations for extra sheets, namely
      the dnd style file: https://github.com/rpgtex/DND-5e-LaTeX-Template.
    debug
      Provide extra info and preserve temporary files.

    """
    # Load properties from file
    if character is None:
        character_props = readers.read_sheet_file(char_file)
        character = _char.Character.load(character_props)
    # Load image file if present
    portrait_file=""
    if character.portrait:
        portrait_file=char_file.stem + ".jpeg"
    # Set the fields in the FDF
    basename = char_file.stem
    char_base = basename + "_char"
    person_base = basename + "_person"
    sheets = [char_base + ".pdf", person_base + ".pdf"]
    pages = []
    # Prepare the tex/html content
    content_suffix = format_suffixes[output_format]
    # Create a list of features and magic items
    content = make_character_content(character=character,
                                     content_format=content_suffix,
                                     fancy_decorations=fancy_decorations)
    # Typeset combined LaTeX file
    if output_format == "pdf":
        # Fillable PDF forms
        char_pdf = create_character_pdf_template(
            character=character, basename=char_base, flatten=flatten
        )
        pages.append(char_pdf)
        person_pdf = create_personality_pdf_template(
            character=character, basename=person_base, portrait_file=portrait_file, flatten=flatten
        )
        pages.append(person_pdf)
        if character.is_spellcaster:
            # Create spell sheet
            spell_base = "{:s}_spells".format(basename)
            create_spells_pdf_template(
                character=character, basename=spell_base, flatten=flatten
            )
            sheets.append(spell_base + ".pdf")
        # Combined with additional LaTeX pages with detailed character info
        features_base = "{:s}_features".format(basename)
        try:
            if len(content) > 2:
                latex.create_latex_pdf(
                    tex="".join(content),
                    basename=features_base,
                    keep_temp_files=debug,
                    use_dnd_decorations=fancy_decorations,
                )
                sheets.append(features_base + ".pdf")
                final_pdf = f"{basename}.pdf"
                merge_pdfs(sheets, final_pdf, clean_up=True)
        except exceptions.LatexNotFoundError:
            log.warning(
                f"``pdflatex`` not available. Skipping features for {character.name}"
            )
    elif output_format == "epub":
        epub.create_epub(
            chapters={character.name: "".join(content)},
            basename=basename,
            title=character.name,
            use_dnd_decorations=fancy_decorations,
        )            
    else:
        raise exceptions.UnknownOutputFormat(
            f"Unknown output format requested: {output_format}. Valid options are:"
            " 'pdf', 'epub'"
        )


def merge_pdfs(src_filenames, dest_filename, clean_up=False):
    """Merge several PDF files into a single final file.

    src_filenames
      Iterable of source PDF file paths to use.
    dest_filename
      Path to requested PDF filename, will be overwritten if it
      exists.
    clean_up : optional
      If truthy, the ``src_filenames`` will be deleted once the
      ``dest_filename`` has been created.

    """
    popenargs = (PDFTK_CMD, *src_filenames, "cat", "output", dest_filename)
    try:
        subprocess.call(popenargs)
    except FileNotFoundError:
        warnings.warn(
            f"Could not run `{PDFTK_CMD}`; skipping file concatenation.", RuntimeWarning
        )
    else:
        # Remove temporary files
        if clean_up:
            for sheet in src_filenames:
                os.remove(sheet)


def _build(filename, args) -> int:
    basename = filename.stem
    print(f"Processing {basename}...")
    try:
        make_sheet(
            sheet_file=filename,
            flatten=(not args.editable),
            output_format=args.output_format,
            debug=args.debug,
            fancy_decorations=args.fancy_decorations,
        )
    except exceptions.CharacterFileFormatError:
        # Only raise the failed exception if this file is explicitly given
        print(f"invalid {basename}")
        if args.filename:
            raise
    except Exception:
        print(f"{basename} failed")
        raise
    else:
        print(f"{basename} done")
    return 1


def main(args=None):
    # Prepare an argument parser
    parser = argparse.ArgumentParser(
        description="Prepare Dungeons and Dragons character sheets as PDFs"
    )
    parser.add_argument(
        "filename",
        type=str,
        nargs="*",
        help="File with character definition, or directory containing such files",
    )
    parser.add_argument(
        "--editable",
        "-e",
        action="store_true",
        help="Keep the PDF fields in place once processed",
    )
    parser.add_argument(
        "--recursive",
        "-r",
        action="store_true",
        help="Descend into subfolders looking for character files",
    )
    parser.add_argument(
        "--fancy-decorations",
        "--fancy",
        "-F",
        action="store_true",
        help=(
            "Render extra pages using fancy decorations "
            "(experimental, requires https://github.com/rpgtex/DND-5e-LaTeX-Template)"
        ),
    )
    parser.add_argument(
        "--output-format",
        "-o",
        help="Specify the output format for the sheets.",
        choices=["pdf", "epub"],
        default="pdf",
    )
    parser.add_argument(
        "--debug",
        "-d",
        action="store_true",
        help="Provide verbose logging for debugging purposes.",
    )
    args = parser.parse_args(args)
    # Prepare logging if necessary
    if args.debug:
        logging.basicConfig(level=logging.DEBUG)
    # Build the true list of filenames
    input_filenames = args.filename
    known_extensions = readers.readers_by_extension.keys()
    if input_filenames == []:
        input_filenames = [Path()]
    else:
        input_filenames = [Path(f) for f in input_filenames]

    def get_char_files(fpath, parse_dirs=False):
        valid_files = []
        if fpath.is_dir() and parse_dirs:
            for f in fpath.iterdir():
                valid_files.extend(get_char_files(f, parse_dirs=args.recursive))
        elif fpath.suffix in known_extensions:
            valid_files.append(fpath)
        else:
            log.info(f"Unhandled file: {str(fpath)}")
        return valid_files

    temp_filenames = []
    for fpath in input_filenames:
        temp_filenames.extend(get_char_files(fpath, parse_dirs=True))
    # IMPORANT:
    # Check that the files are valid dungeonsheets files without importing them
    filenames = []
    version_re = re.compile(
        r"^dungeonsheets_version = [\'\"](?P<version>[0-9.]+)[\'\"]\s*$", re.MULTILINE
    )
    for fpath in temp_filenames:
        with open(fpath, mode="r") as fp:
            if version_re.search(fp.read()) or fpath.suffix != ".py":
                filenames.append(fpath)
    # Process the requested files
    if args.debug:
        for filename in filenames:
            log.debug("building")
            _build(filename, args)
    else:
        with Pool(cpu_count()) as p:
            p.starmap(_build, product(filenames, [args]))


if __name__ == "__main__":
    main()



"""
A collection of monsters from D&D 5e's SRD

This file was autogenerated by https://github.com/stravajiaxen/5e-srd-to-py
"""

from dungeonsheets.monsters.monsters import Monster
from dungeonsheets.stats import Ability


class Baboon(Monster):
    """
    Pack Tactics.
      The baboon has advantage on an attack roll against a creature if at
      least one of the baboon's allies is within 5 ft. of the creature and
      the ally isn't incapacitated.
    Bite.
      Melee Weapon Attack: +1 to hit, reach 5 ft., one target. Hit: 1 (1d4 -
      1) piercing damage.
    """
    name = "Baboon"
    description = "Small beast, unaligned"
    challenge_rating = 0
    armor_class = 12
    skills = ""
    senses = "Passive Perception 11"
    languages = ""
    strength = Ability(8)
    dexterity = Ability(14)
    constitution = Ability(11)
    intelligence = Ability(4)
    wisdom = Ability(12)
    charisma = Ability(6)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 30
    hp_max = 3
    hit_dice = "1d6"
    spells = []


class Badger(Monster):
    """
    Keen Smell.
      The badger has advantage on Wisdom (Perception) checks that rely on
      smell.
    Bite.
      Melee Weapon Attack: +2 to hit, reach 5 ft., one target. Hit: 1
      piercing damage.
    """
    name = "Badger"
    description = "Tiny beast, unaligned"
    challenge_rating = 0
    armor_class = 10
    skills = ""
    senses = "Darkvision 30 ft., Passive Perception 11"
    languages = ""
    strength = Ability(4)
    dexterity = Ability(11)
    constitution = Ability(12)
    intelligence = Ability(2)
    wisdom = Ability(12)
    charisma = Ability(5)
    speed = 20
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 3
    hit_dice = "1d4"
    spells = []


class Balor(Monster):
    """
    Death Throes.
      When the balor dies, it explodes, and each creature within 30 feet of
      it must make a DC 20 Dexterity saving throw, taking 70 (20d6) fire
      damage on a failed save, or half as much damage on a successful one.
      The explosion ignites flammable objects in that area that aren't being
      worn or carried, and it destroys the balor's weapons.
    Fire Aura.
      At the start of each of the balor's turns, each creature within 5 feet
      of it takes 10 (3d6) fire damage, and flammable objects in the aura
      that aren't being worn or carried ignite. A creature that touches the
      balor or hits it with a melee attack while within 5 feet of it takes
      10 (3d6) fire damage.
    Magic Resistance.
      The balor has advantage on saving throws against spells and other
      magical effects.
    Magic Weapons.
      The balor's weapon attacks are magical.
    Multiattack.
      The balor makes two attacks: one with its longsword and one with its
      whip.
    Longsword.
      Melee Weapon Attack: +14 to hit, reach 10 ft., one target. Hit: 21
      (3d8 + 8) slashing damage plus 13 (3d8) lightning damage. If the balor
      scores a critical hit, it rolls damage dice three times, instead of
      twice.
    Whip.
      Melee Weapon Attack: +14 to hit, reach 30 ft., one target. Hit: 15
      (2d6 + 8) slashing damage plus 10 (3d6) fire damage, and the target
      must succeed on a DC 20 Strength saving throw or be pulled up to 25
      feet toward the balor.
    Teleport.
      The balor magically teleports, along with any equipment it is wearing
      or carrying, up to 120 feet to an unoccupied space it can see.
    Variant: Summon Demon.
      The demon chooses what to summon and attempts a magical summoning.
      
      A balor has a 50 percent chance of summoning 1d8 vrocks, 1d6 hezrous,
      1d4 glabrezus, 1d3 nalfeshnees, 1d2 mariliths, or one goristro.
      
      A summoned demon appears in an unoccupied space within 60 feet of its
      summoner, acts as an ally of its summoner, and can't summon other
      demons. It remains for 1 minute, until it or its summoner dies, or
      until its summoner dismisses it as an action.
    """
    name = "Balor"
    description = "Huge fiend, chaotic evil"
    challenge_rating = 19
    armor_class = 19
    skills = ""
    senses = "Truesight 120 ft., Passive Perception 13"
    languages = "Abyssal, telepathy 120 ft."
    strength = Ability(26)
    dexterity = Ability(15)
    constitution = Ability(22)
    intelligence = Ability(20)
    wisdom = Ability(16)
    charisma = Ability(22)
    speed = 40
    swim_speed = 0
    fly_speed = 80
    climb_speed = 0
    hp_max = 262
    hit_dice = "21d12"
    spells = []


class Bandit(Monster):
    """
    Scimitar.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 4 (1d6 +
      1) slashing damage.
    Light Crossbow.
      Ranged Weapon Attack: +3 to hit, range 80 ft./320 ft., one target.
      Hit: 5 (1d8 + 1) piercing damage.
    """
    name = "Bandit"
    description = "Medium humanoid, any non-lawful alignment"
    challenge_rating = 0.125
    armor_class = 12
    skills = ""
    senses = "Passive Perception 10"
    languages = "any one language (usually Common)"
    strength = Ability(11)
    dexterity = Ability(12)
    constitution = Ability(12)
    intelligence = Ability(10)
    wisdom = Ability(10)
    charisma = Ability(10)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 11
    hit_dice = "2d8"
    spells = []


class BanditCaptain(Monster):
    """
    Multiattack.
      The captain makes three melee attacks: two with its scimitar and one
      with its dagger. Or the captain makes two ranged attacks with its
      daggers.
    Scimitar.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 6 (1d6 +
      3) slashing damage.
    Dagger.
      Melee or Ranged Weapon Attack: +5 to hit, reach 5 ft. or range 20/60
      ft., one target. Hit: 5 (1d4 + 3) piercing damage.
    """
    name = "Bandit Captain"
    description = "Medium humanoid, any non-lawful alignment"
    challenge_rating = 2
    armor_class = 15
    skills = "Athletics +4, Deception +4"
    senses = "Passive Perception 10"
    languages = "any two languages"
    strength = Ability(15)
    dexterity = Ability(16)
    constitution = Ability(14)
    intelligence = Ability(14)
    wisdom = Ability(11)
    charisma = Ability(14)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 65
    hit_dice = "10d8"
    spells = []


class BarbedDevil(Monster):
    """
    Barbed Hide.
      At the start of each of its turns, the barbed devil deals 5 (1d10)
      piercing damage to any creature grappling it.
    Devil's Sight.
      Magical darkness doesn't impede the devil's darkvision.
    Magic Resistance.
      The devil has advantage on saving throws against spells and other
      magical effects.
    Multiattack.
      The devil makes three melee attacks: one with its tail and two with
      its claws. Alternatively, it can use Hurl Flame twice.
    Claw.
      Melee Weapon Attack: +6 to hit, reach 5 ft ., one target. Hit: 6 (1d6
      + 3) piercing damage.
    Tail.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 10 (2d6
      + 3) piercing damage.
    Hurl Flame.
      Ranged Spell Attack: +5 to hit, range 150 ft., one target. Hit: 10
      (3d6) fire damage. If the target is a flammable object that isn't
      being worn or carried, it also catches fire.
    """
    name = "Barbed Devil"
    description = "Medium fiend, lawful evil"
    challenge_rating = 5
    armor_class = 15
    skills = "Deception +5, Insight +5, Perception +8"
    senses = "Darkvision 120 ft., Passive Perception 18"
    languages = "Infernal, telepathy 120 ft."
    strength = Ability(16)
    dexterity = Ability(17)
    constitution = Ability(18)
    intelligence = Ability(12)
    wisdom = Ability(14)
    charisma = Ability(14)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 110
    hit_dice = "13d8"
    spells = []


class Basilisk(Monster):
    """
    Petrifying Gaze.
      If a creature starts its turn within 30 ft. of the basilisk and the
      two of them can see each other, the basilisk can force the creature to
      make a DC 12 Constitution saving throw if the basilisk isn't
      incapacitated. On a failed save, the creature magically begins to turn
      to stone and is restrained. It must repeat the saving throw at the end
      of its next turn. On a success, the effect ends. On a failure, the
      creature is petrified until freed by the greater restoration spell or
      other magic.
      
      A creature that isn't surprised can avert its eyes to avoid the saving
      throw at the start of its turn. If it does so, it can't see the
      basilisk until the start of its next turn, when it can avert its eyes
      again. If it looks at the basilisk in the meantime, it must
      immediately make the save.
      
      If the basilisk sees its reflection within 30 ft. of it in bright
      light, it mistakes itself for a rival and targets itself with its
      gaze.
    Bite.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 10 (2d6
      + 3) piercing damage plus 7 (2d6) poison damage.
    """
    name = "Basilisk"
    description = "Medium monstrosity, unaligned"
    challenge_rating = 3
    armor_class = 12
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 9"
    languages = ""
    strength = Ability(16)
    dexterity = Ability(8)
    constitution = Ability(15)
    intelligence = Ability(2)
    wisdom = Ability(8)
    charisma = Ability(7)
    speed = 20
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 52
    hit_dice = "8d8"
    spells = []


class Bat(Monster):
    """
    Echolocation.
      The bat can't use its blindsight while deafened.
    Keen Hearing.
      The bat has advantage on Wisdom (Perception) checks that rely on
      hearing.
    Bite.
      Melee Weapon Attack: +0 to hit, reach 5 ft., one creature. Hit: 1
      piercing damage.
    """
    name = "Bat"
    description = "Tiny beast, unaligned"
    challenge_rating = 0
    armor_class = 12
    skills = ""
    senses = "Blindsight 60 ft., Passive Perception 11"
    languages = ""
    strength = Ability(2)
    dexterity = Ability(15)
    constitution = Ability(8)
    intelligence = Ability(2)
    wisdom = Ability(12)
    charisma = Ability(4)
    speed = 5
    swim_speed = 0
    fly_speed = 30
    climb_speed = 0
    hp_max = 1
    hit_dice = "1d4"
    spells = []


class BeardedDevil(Monster):
    """
    Devil's Sight.
      Magical darkness doesn't impede the devil's darkvision.
    Magic Resistance.
      The devil has advantage on saving throws against spells and other
      magical effects.
    Steadfast.
      The devil can't be frightened while it can see an allied creature
      within 30 feet of it.
    Multiattack.
      The devil makes two attacks: one with its beard and one with its
      glaive.
    Beard.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one creature. Hit: 6 (1d8
      + 2) piercing damage, and the target must succeed on a DC 12
      Constitution saving throw or be poisoned for 1 minute. While poisoned
      in this way, the target can't regain hit points. The target can repeat
      the saving throw at the end of each of its turns, ending the effect on
      itself on a success.
    Glaive.
      Melee Weapon Attack: +5 to hit, reach 10 ft., one target. Hit: 8 (1d10
      + 3) slashing damage. If the target is a creature other than an undead
      or a construct, it must succeed on a DC 12 Constitution saving throw
      or lose 5 (1d10) hit points at the start of each of its turns due to
      an infernal wound. Each time the devil hits the wounded target with
      this attack, the damage dealt by the wound increases by 5 (1d10). Any
      creature can take an action to stanch the wound with a successful DC
      12 Wisdom (Medicine) check. The wound also closes if the target
      receives magical healing.
    """
    name = "Bearded Devil"
    description = "Medium fiend, lawful evil"
    challenge_rating = 3
    armor_class = 13
    skills = ""
    senses = "Darkvision 120 ft., Passive Perception 10"
    languages = "Infernal, telepathy 120 ft."
    strength = Ability(16)
    dexterity = Ability(15)
    constitution = Ability(15)
    intelligence = Ability(9)
    wisdom = Ability(11)
    charisma = Ability(11)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 52
    hit_dice = "8d8"
    spells = []


class Behir(Monster):
    """
    Multiattack.
      The behir makes two attacks: one with its bite and one to constrict.
    Bite.
      Melee Weapon Attack: +10 to hit, reach 10 ft., one target. Hit: 22
      (3d10 + 6) piercing damage.
    Constrict.
      Melee Weapon Attack: +10 to hit, reach 5 ft., one Large or smaller
      creature. Hit: 17 (2d10 + 6) bludgeoning damage plus 17 (2d10 + 6)
      slashing damage. The target is grappled (escape DC 16) if the behir
      isn't already constricting a creature, and the target is restrained
      until this grapple ends.
    Lightning Breath.
      The behir exhales a line of lightning that is 20 ft. long and 5 ft.
      wide. Each creature in that line must make a DC 16 Dexterity saving
      throw, taking 66 (12d10) lightning damage on a failed save, or half as
      much damage on a successful one.
    Swallow.
      The behir makes one bite attack against a Medium or smaller target it
      is grappling. If the attack hits, the target is also swallowed, and
      the grapple ends. While swallowed, the target is blinded and
      restrained, it has total cover against attacks and other effects
      outside the behir, and it takes 21 (6d6) acid damage at the start of
      each of the behir's turns. A behir can have only one creature
      swallowed at a time.
      
      If the behir takes 30 damage or more on a single turn from the
      swallowed creature, the behir must succeed on a DC 14 Constitution
      saving throw at the end of that turn or regurgitate the creature,
      which falls prone in a space within 10 ft. of the behir. If the behir
      dies, a swallowed creature is no longer restrained by it and can
      escape from the corpse by using 15 ft. of movement, exiting prone.
    """
    name = "Behir"
    description = "Huge monstrosity, neutral evil"
    challenge_rating = 11
    armor_class = 17
    skills = "Perception +6, Stealth +7"
    senses = "Darkvision 90 ft., Passive Perception 16"
    languages = "Draconic"
    strength = Ability(23)
    dexterity = Ability(16)
    constitution = Ability(18)
    intelligence = Ability(7)
    wisdom = Ability(14)
    charisma = Ability(12)
    speed = 50
    swim_speed = 0
    fly_speed = 0
    climb_speed = 40
    hp_max = 168
    hit_dice = "16d12"
    spells = []


class Berserker(Monster):
    """
    Reckless.
      At the start of its turn, the berserker can gain advantage on all
      melee weapon attack rolls during that turn, but attack rolls against
      it have advantage until the start of its next turn.
    Greataxe.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 9 (1d12
      + 3) slashing damage.
    """
    name = "Berserker"
    description = "Medium humanoid, any chaotic alignment"
    challenge_rating = 2
    armor_class = 13
    skills = ""
    senses = "Passive Perception 10"
    languages = "any one language (usually Common)"
    strength = Ability(16)
    dexterity = Ability(12)
    constitution = Ability(17)
    intelligence = Ability(9)
    wisdom = Ability(11)
    charisma = Ability(9)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 67
    hit_dice = "9d8"
    spells = []


class BlackBear(Monster):
    """
    Keen Smell.
      The bear has advantage on Wisdom (Perception) checks that rely on
      smell.
    Multiattack.
      The bear makes two attacks: one with its bite and one with its claws.
    Bite.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 5 (1d6 +
      2) piercing damage.
    Claws.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 7 (2d4 +
      2) slashing damage.
    """
    name = "Black Bear"
    description = "Medium beast, unaligned"
    challenge_rating = 0.5
    armor_class = 11
    skills = ""
    senses = "Passive Perception 13"
    languages = ""
    strength = Ability(15)
    dexterity = Ability(10)
    constitution = Ability(14)
    intelligence = Ability(2)
    wisdom = Ability(12)
    charisma = Ability(7)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 30
    hp_max = 19
    hit_dice = "3d8"
    spells = []


class BlackDragonWyrmling(Monster):
    """
    Amphibious.
      The dragon can breathe air and water.
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 7 (1d10
      + 2) piercing damage plus 2 (1d4) acid damage.
    Acid Breath.
      The dragon exhales acid in a 15-foot line that is 5 feet wide. Each
      creature in that line must make a DC 11 Dexterity saving throw, taking
      22 (Sd8) acid damage on a failed save, or half as much damage on a
      successful one.
    """
    name = "Black Dragon Wyrmling"
    description = "Medium dragon, chaotic evil"
    challenge_rating = 2
    armor_class = 17
    skills = "Perception +4, Stealth +4"
    senses = "Blindsight 10 ft., Darkvision 60 ft., Passive Perception 14"
    languages = "Draconic"
    strength = Ability(15)
    dexterity = Ability(14)
    constitution = Ability(13)
    intelligence = Ability(10)
    wisdom = Ability(11)
    charisma = Ability(13)
    speed = 30
    swim_speed = 30
    fly_speed = 60
    climb_speed = 0
    hp_max = 33
    hit_dice = "6d8"
    spells = []


class BlackPudding(Monster):
    """
    Amorphous.
      The pudding can move through a space as narrow as 1 inch wide without
      squeezing.
    Corrosive Form.
      A creature that touches the pudding or hits it with a melee attack
      while within 5 feet of it takes 4 (1d8) acid damage. Any nonmagical
      weapon made of metal or wood that hits the pudding corrodes. After
      dealing damage, the weapon takes a permanent and cumulative -1 penalty
      to damage rolls. If its penalty drops to -5, the weapon is destroyed.
      Nonmagical ammunition made of metal or wood that hits the pudding is
      destroyed after dealing damage. The pudding can eat through 2-inch-
      thick, nonmagical wood or metal in 1 round.
    Spider Climb.
      The pudding can climb difficult surfaces, including upside down on
      ceilings, without needing to make an ability check.
    Pseudopod.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 6 (1d6 +
      3) bludgeoning damage plus 18 (4d8) acid damage. In addition,
      nonmagical armor worn by the target is partly dissolved and takes a
      permanent and cumulative -1 penalty to the AC it offers. The armor is
      destroyed if the penalty reduces its AC to 10.
    """
    name = "Black Pudding"
    description = "Large ooze, unaligned"
    challenge_rating = 4
    armor_class = 7
    skills = ""
    senses = "Blindsight 60 ft. (blind beyond this radius), Passive Perception 8"
    languages = ""
    strength = Ability(16)
    dexterity = Ability(5)
    constitution = Ability(16)
    intelligence = Ability(1)
    wisdom = Ability(6)
    charisma = Ability(1)
    speed = 20
    swim_speed = 0
    fly_speed = 0
    climb_speed = 20
    hp_max = 85
    hit_dice = "10d10"
    spells = []


class BlinkDog(Monster):
    """
    Keen Hearing and Smell.
      The dog has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Bite.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 4 (1d6 +
      1) piercing damage.
    Teleport.
      The dog magically teleports, along with any equipment it is wearing or
      carrying, up to 40 ft. to an unoccupied space it can see. Before or
      after teleporting, the dog can make one bite attack.
    """
    name = "Blink Dog"
    description = "Medium fey, lawful good"
    challenge_rating = 0.25
    armor_class = 13
    skills = "Perception +3, Stealth +5"
    senses = "Passive Perception 10"
    languages = "Blink Dog, understands Sylvan but can't speak it"
    strength = Ability(12)
    dexterity = Ability(17)
    constitution = Ability(12)
    intelligence = Ability(10)
    wisdom = Ability(13)
    charisma = Ability(11)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 22
    hit_dice = "4d8"
    spells = []


class BloodHawk(Monster):
    """
    Keen Sight.
      The hawk has advantage on Wisdom (Perception) checks that rely on
      sight.
    Pack Tactics.
      The hawk has advantage on an attack roll against a creature if at
      least one of the hawk's allies is within 5 ft. of the creature and the
      ally isn't incapacitated.
    Beak.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 4 (1d4 +
      2) piercing damage.
    """
    name = "Blood Hawk"
    description = "Small beast, unaligned"
    challenge_rating = 0.125
    armor_class = 12
    skills = "Perception +4"
    senses = "Passive Perception 14"
    languages = ""
    strength = Ability(6)
    dexterity = Ability(14)
    constitution = Ability(10)
    intelligence = Ability(3)
    wisdom = Ability(14)
    charisma = Ability(5)
    speed = 10
    swim_speed = 0
    fly_speed = 60
    climb_speed = 0
    hp_max = 7
    hit_dice = "2d6"
    spells = []


class BlueDragonWyrmling(Monster):
    """
    Bite.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 8 (1d10
      + 3) piercing damage plus 3 (1d6) lightning damage.
    Lightning Breath.
      The dragon exhales lightning in a 30-foot line that is 5 feet wide.
      Each creature in that line must make a DC 12 Dexterity saving throw,
      taking 22 (4d10) lightning damage on a failed save, or half as much
      damage on a successful one.
    """
    name = "Blue Dragon Wyrmling"
    description = "Medium dragon, lawful evil"
    challenge_rating = 3
    armor_class = 17
    skills = "Perception +4, Stealth +2"
    senses = "Blindsight 10 ft., Darkvision 60 ft., Passive Perception 14"
    languages = "Draconic"
    strength = Ability(17)
    dexterity = Ability(10)
    constitution = Ability(15)
    intelligence = Ability(12)
    wisdom = Ability(11)
    charisma = Ability(15)
    speed = 30
    swim_speed = 0
    fly_speed = 60
    climb_speed = 0
    hp_max = 52
    hit_dice = "8d8"
    spells = []


class Boar(Monster):
    """
    Charge.
      If the boar moves at least 20 ft. straight toward a target and then
      hits it with a tusk attack on the same turn, the target takes an extra
      3 (1d6) slashing damage. If the target is a creature, it must succeed
      on a DC 11 Strength saving throw or be knocked prone.
    Relentless.
      If the boar takes 7 damage or less that would reduce it to 0 hit
      points, it is reduced to 1 hit point instead.
    Tusk.
      Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 4 (1d6 + 1)
      slashing damage.
    """
    name = "Boar"
    description = "Medium beast, unaligned"
    challenge_rating = 0.25
    armor_class = 11
    skills = ""
    senses = "Passive Perception 9"
    languages = ""
    strength = Ability(13)
    dexterity = Ability(11)
    constitution = Ability(12)
    intelligence = Ability(2)
    wisdom = Ability(9)
    charisma = Ability(5)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 11
    hit_dice = "2d8"
    spells = []


class BoneDevil(Monster):
    """
    Devil's Sight.
      Magical darkness doesn't impede the devil's darkvision.
    Magic Resistance.
      The devil has advantage on saving throws against spells and other
      magical effects.
    Multiattack.
      The devil makes three attacks: two with its claws and one with its
      sting.
    Claw.
      Melee Weapon Attack: +8 to hit, reach 10 ft., one target. Hit: 8 (1d8
      + 4) slashing damage.
    Sting.
      Melee Weapon Attack: +8 to hit, reach 10 ft., one target. Hit: 13 (2d8
      + 4) piercing damage plus 17 (5d6) poison damage, and the target must
      succeed on a DC 14 Constitution saving throw or become poisoned for 1
      minute. The target can repeat the saving throw at the end of each of
      its turns, ending the effect on itself on a success .
    """
    name = "Bone Devil"
    description = "Large fiend, lawful evil"
    challenge_rating = 12
    armor_class = 19
    skills = "Deception +7, Insight +6"
    senses = "Darkvision 120 ft., Passive Perception 9"
    languages = "Infernal, telepathy 120 ft."
    strength = Ability(18)
    dexterity = Ability(16)
    constitution = Ability(18)
    intelligence = Ability(13)
    wisdom = Ability(14)
    charisma = Ability(16)
    speed = 40
    swim_speed = 0
    fly_speed = 40
    climb_speed = 0
    hp_max = 142
    hit_dice = "15d10"
    spells = []


class BrassDragonWyrmling(Monster):
    """
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 7 (1d10
      + 2) piercing damage.
    Breath Weapons.
      The dragon uses one of the following breath weapons.
      
      Fire Breath. The dragon exhales fire in an 20-foot line that is 5 feet
      wide. Each creature in that line must make a DC 11 Dexterity saving
      throw, taking 14 (4d6) fire damage on a failed save, or half as much
      damage on a successful one.
      
      Sleep Breath. The dragon exhales sleep gas in a 15-foot cone. Each
      creature in that area must succeed on a DC 11 Constitution saving
      throw or fall unconscious for 1 minute. This effect ends for a
      creature if the creature takes damage or someone uses an action to
      wake it.
    """
    name = "Brass Dragon Wyrmling"
    description = "Medium dragon, chaotic good"
    challenge_rating = 1
    armor_class = 16
    skills = "Perception +4, Stealth +2"
    senses = "Blindsight 10 ft., Darkvision 60 ft., Passive Perception 14"
    languages = "Draconic"
    strength = Ability(15)
    dexterity = Ability(10)
    constitution = Ability(13)
    intelligence = Ability(10)
    wisdom = Ability(11)
    charisma = Ability(13)
    speed = 30
    swim_speed = 0
    fly_speed = 60
    climb_speed = 0
    hp_max = 16
    hit_dice = "3d8"
    spells = []


class BronzeDragonWyrmling(Monster):
    """
    Amphibious.
      The dragon can breathe air and water.
    Bite.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 8 (1d10
      + 3) piercing damage.
    Breath Weapons.
      The dragon uses one of the following breath weapons.
      
      Lightning Breath. The dragon exhales lightning in a 40-foot line that
      is 5 feet wide. Each creature in that line must make a DC 12 Dexterity
      saving throw, taking 16 (3d10) lightning damage on a failed save, or
      half as much damage on a successful one.
      
      Repulsion Breath. The dragon exhales repulsion energy in a 30-foot
      cone. Each creature in that area must succeed on a DC 12 Strength
      saving throw. On a failed save, the creature is pushed 30 feet away
      from the dragon.
    """
    name = "Bronze Dragon Wyrmling"
    description = "Medium dragon, lawful good"
    challenge_rating = 2
    armor_class = 17
    skills = "Perception +4, Stealth +2"
    senses = "Blindsight 10 ft., Darkvision 60 ft., Passive Perception 14"
    languages = "Draconic"
    strength = Ability(17)
    dexterity = Ability(10)
    constitution = Ability(15)
    intelligence = Ability(12)
    wisdom = Ability(11)
    charisma = Ability(15)
    speed = 30
    swim_speed = 30
    fly_speed = 60
    climb_speed = 0
    hp_max = 32
    hit_dice = "5d8"
    spells = []


class BrownBear(Monster):
    """
    Keen Smell.
      The bear has advantage on Wisdom (Perception) checks that rely on
      smell.
    Multiattack.
      The bear makes two attacks: one with its bite and one with its claws.
    Bite.
      Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 8 (1d8 + 4)
      piercing damage.
    Claws.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 11 (2d6
      + 4) slashing damage.
    """
    name = "Brown Bear"
    description = "Large beast, unaligned"
    challenge_rating = 1
    armor_class = 11
    skills = "Perception +3"
    senses = "Passive Perception 13"
    languages = ""
    strength = Ability(19)
    dexterity = Ability(10)
    constitution = Ability(16)
    intelligence = Ability(2)
    wisdom = Ability(13)
    charisma = Ability(7)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 30
    hp_max = 34
    hit_dice = "4d10"
    spells = []


class Bugbear(Monster):
    """
    Brute.
      A melee weapon deals one extra die of its damage when the bugbear hits
      with it (included in the attack).
    Surprise Attack.
      If the bugbear surprises a creature and hits it with an attack during
      the first round of combat, the target takes an extra 7 (2d6) damage
      from the attack.
    Morningstar.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 11 (2d8
      + 2) piercing damage.
    Javelin.
      Melee or Ranged Weapon Attack: +4 to hit, reach 5 ft. or range 30/120
      ft., one target. Hit: 9 (2d6 + 2) piercing damage in melee or 5 (1d6 +
      2) piercing damage at range.
    """
    name = "Bugbear"
    description = "Medium humanoid, chaotic evil"
    challenge_rating = 1
    armor_class = 16
    skills = "Stealth +6, Survival +2"
    senses = "Darkvision 60 ft., Passive Perception 10"
    languages = "Common, Goblin"
    strength = Ability(15)
    dexterity = Ability(14)
    constitution = Ability(13)
    intelligence = Ability(8)
    wisdom = Ability(11)
    charisma = Ability(9)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 27
    hit_dice = "5d8"
    spells = []


# PotA
class Burrowshark(Monster):
    """Elite warriors of the earth cult, burrowsharks are fierce champions
    who ride trained bulettes into battle. While their powerful mounts
    rend and tear foes to pieces, burrowsharks leap to the ground and
    cut down their foes without mercy.
    
    Burrowsharks are much like Black Earth guards, since both have
    uncanny footing and special armor. For burrowsharks, an additional
    gift of Ogremoch's might establishes a magical bond between the
    burrowshark and a bulette, allowing the rider to burrow with its
    mount and sense what its mount senses.
    
    Bond of the Black Earth
      The burrowshark is magically bound to a bulette trained to serve
      as its mount. While mounted on its bulette, the burrowshark
      shares the bulette's sense and can ride the bulette while it
      burrows. The bonded bulette obeys the burrowshark's commands. If
      its mount dies, the burrowshark can train a new bulette to serve
      as its bonded mount, a process requiring a month.
    Multiattack
      The burrowshark makes three melee attacks.
    Spear
      Melee or ranged weapon attack: +6 to hit, reach 5ft. or range
      20/60ft., one target. Hit: 7 (1d6+4) piercing damage, or 8
      (1d8+4) piercing damage if used with two hands to make a melee
      attack.
    Unyielding
      When the burrowshark is subjected to an effect that would move
      it, knock it prone, or both, it can use its reaction to be
      neither moved nor knocked prone.
    
    """
    name = "Burrowshark"
    description = "Medium humanoid (human), neutral evil"
    challenge_rating = 4
    armor_class = 18
    skills = "Animal handling +2, Athletics +6, Intimidation + 3, Perception +2"
    senses = "passive Perception 12"
    languages = "Common"
    strength = Ability(18)
    dexterity = Ability(12)
    constitution = Ability(16)
    intelligence = Ability(10)
    wisdom = Ability(11)
    charisma = Ability(13)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 82
    hit_dice = "11d8+33"


class Bulette(Monster):
    """
    Standing Leap.
      The bulette's long jump is up to 30 ft. and its high jump is up to 15
      ft., with or without a running start.
    Bite.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 30 (4d12
      + 4) piercing damage.
    Deadly Leap.
      If the bulette jumps at least 15 ft. as part of its movement, it can
      then use this action to land on its ft. in a space that contains one
      or more other creatures. Each of those creatures must succeed on a DC
      16 Strength or Dexterity saving throw (target's choice) or be knocked
      prone and take 14 (3d6 + 4) bludgeoning damage plus 14 (3d6 + 4)
      slashing damage. On a successful save, the creature takes only half
      the damage, isn't knocked prone, and is pushed 5 ft. out of the
      bulette's space into an unoccupied space of the creature's choice. If
      no unoccupied space is within range, the creature instead falls prone
      in the bulette's space.
    """
    name = "Bulette"
    description = "Large monstrosity, unaligned"
    challenge_rating = 5
    armor_class = 17
    skills = "Perception +6"
    senses = "Darkvision 60 ft., Tremorsense 60 ft., Passive Perception 16"
    languages = ""
    strength = Ability(19)
    dexterity = Ability(11)
    constitution = Ability(21)
    intelligence = Ability(2)
    wisdom = Ability(10)
    charisma = Ability(5)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 94
    hit_dice = "9d10"
    spells = []



"""
A collection of monsters from D&D 5e's SRD

This file was autogenerated by https://github.com/stravajiaxen/5e-srd-to-py
"""

from dungeonsheets.monsters.monsters import Monster
from dungeonsheets.stats import Ability


class Dao(Monster):
    """Earth Glide.
      The dao can burrow through nonmagical unworked earth and
      stone. While doing so, the dao doesn't distrub the material it
      moves through.
    Elemental Demise.
      If the dao dies, its body disintegrates into crystalline powder,
      leaving behind only equipment the dao was wearing or carrying.
    Innate Spellcasting.
      The dao's innate spellcasting ability is Charisma (spell save DC
      14, +6 to hit with spell attacks). It can innately cast the
      following spells, requiring no material components:
      
      - At will: detect evil and good, detect magic, stone shape.
      - 3/day each: passwall, move earth, tongues.
      - 1/day each: conjure elemental (earth elemental only), gaseous
        form, invisibility, phantasmal killer, plane shift, wall of
        stone.
      
    Sure-Footed.
      The dao has advantage on Strength and Dexterity saving throws
      made against effects that would knock it prone.
    Multiattack.
      The dao makes two fist attacks or two maul attacks.
    Fist.
      *Melee weapon attack:* +10 to hit, reach 5ft., one
      target. *Hit:* 15 (2d8 + 6) bludgeoning damage.
    Maul.
      *Melee weapon attack:* +10 to hit, reach 5ft., one
      target. *Hit:* 20 (4d6 + 6) bludgeoning damage. If the target is
      a Huge or smaller creature, it must succeed on a DC 18 Strength
      check or be knocked prone.

    """
    name = "Dao"
    description = "Large elemental, neutral evil"
    challenge_rating = 11
    armor_class = 18
    skills = ""
    saving_throws = "Int +5, Wis +5, Cha +6"
    condition_immunities = "petrified"
    senses = "darkvision 120ft., passive Perception 11"
    languages = "Terran"
    strength = Ability(23)
    dexterity = Ability(12)
    constitution = Ability(24)
    intelligence = Ability(12)
    wisdom = Ability(13)
    charisma = Ability(14)
    speed = 30
    swim_speed = 0
    fly_speed = 30
    climb_speed = 0
    burrow_speed = 30
    hp_max = 187
    hit_dice = "15d10 + 105"
    spells = ["detect evil and good", "detect magic", "stone shape",
              "passwall", "move earth", "tongues",
              "conjure elemental", "gaseous form",
              "invisibility", "phantasmal killer", "plane shift",
              "wall of stone"]


class Darkmantle(Monster):
    """
    Echolocation.
      The darkmantle can't use its blindsight while deafened.
    False Appearance.
      While the darkmantle remains motionless, it is indistinguishable from
      a cave formation such as a stalactite or stalagmite.
    Crush.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one creature. Hit: 6 (1d6
      + 3) bludgeoning damage, and the darkmantle attaches to the target. If
      the target is Medium or smaller and the darkmantle has advantage on
      the attack roll, it attaches by engulfing the target's head, and the
      target is also blinded and unable to breathe while the darkmantle is
      attached in this way.
      
      While attached to the target, the darkmantle can attack no other
      creature except the target but has advantage on its attack rolls. The
      darkmantle's speed also becomes 0, it can't benefit from any bonus to
      its speed, and it moves with the target.
      
      A creature can detach the darkmantle by making a successful DC 13
      Strength check as an action. On its turn, the darkmantle can detach
      itself from the target by using 5 feet of movement.
    Darkness Aura.
      A 15-foot radius of magical darkness extends out from the darkmantle,
      moves with it, and spreads around corners. The darkness lasts as long
      as the darkmantle maintains concentration, up to 10 minutes (as if
      concentrating on a spell). Darkvision can't penetrate this darkness,
      and no natural light can illuminate it. If any of the darkness
      overlaps with an area of light created by a spell of 2nd level or
      lower, the spell creating the light is dispelled.
    """
    name = "Darkmantle"
    description = "Small monstrosity, unaligned"
    challenge_rating = 0.5
    armor_class = 11
    skills = "Stealth +3"
    senses = "Blindsight 60 ft., Passive Perception 10"
    languages = ""
    strength = Ability(16)
    dexterity = Ability(12)
    constitution = Ability(13)
    intelligence = Ability(2)
    wisdom = Ability(10)
    charisma = Ability(5)
    speed = 10
    swim_speed = 0
    fly_speed = 30
    climb_speed = 0
    hp_max = 22
    hit_dice = "5d6"
    spells = []


class DeathDog(Monster):
    """
    Two-Headed.
      The dog has advantage on Wisdom (Perception) checks and on saving
      throws against being blinded, charmed, deafened, frightened, stunned,
      or knocked unconscious.
    Multiattack.
      The dog makes two bite attacks.
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 5 (1d6 +
      2) piercing damage. If the target is a creature, it must succeed on a
      DC 12 Constitution saving throw against disease or become poisoned
      until the disease is cured. Every 24 hours that elapse, the creature
      must repeat the saving throw, reducing its hit point maximum by 5
      (1d10) on a failure. This reduction lasts until the disease is cured.
      The creature dies if the disease reduces its hit point maximum to 0.
    """
    name = "Death Dog"
    description = "Medium monstrosity, neutral evil"
    challenge_rating = 1
    armor_class = 12
    skills = "Perception +5, Stealth +4"
    senses = "Darkvision 120 ft., Passive Perception 15"
    languages = ""
    strength = Ability(15)
    dexterity = Ability(14)
    constitution = Ability(14)
    intelligence = Ability(3)
    wisdom = Ability(13)
    charisma = Ability(6)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 39
    hit_dice = "6d8"
    spells = []


class DeepGnomeSvirfneblin(Monster):
    """
    Stone Camouflage.
      The gnome has advantage on Dexterity (Stealth) checks made to hide in
      rocky terrain.
    Gnome Cunning.
      The gnome has advantage on Intelligence, Wisdom, and Charisma saving
      throws against magic.
    Innate Spellcasting.
      The gnome's innate spellcasting ability is Intelligence (spell save DC
      11). It can innately cast the following spells, requiring no material
      components:
      
      At will: nondetection (self only)
      
      1/day each: blindness/deafness, blur, disguise self
    War Pick.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 6 (1d8 +
      2) piercing damage.
    Poisoned Dart.
      Ranged Weapon Attack: +4 to hit, range 30/120 ft., one creature. Hit:
      4 (1d4 + 2) piercing damage, and the target must succeed on a DC 12
      Constitution saving throw or be poisoned for 1 minute. The target can
      repeat the saving throw at the end of each of its turns, ending the
      effect on itself on a success
    """
    name = "Deep Gnome (Svirfneblin)"
    description = "Small humanoid, neutral good"
    challenge_rating = 0.5
    armor_class = 15
    skills = "Investigation +3, Perception +2, Stealth +4"
    senses = "Darkvision 120 ft., Passive Perception 12"
    languages = "Gnomish, Terran, Undercommon"
    strength = Ability(15)
    dexterity = Ability(14)
    constitution = Ability(14)
    intelligence = Ability(12)
    wisdom = Ability(10)
    charisma = Ability(9)
    speed = 20
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 16
    hit_dice = "3d6"
    spells = []


class DeepGnomeSvirfneblin(Monster):
    """Stone Camouflage.
      The gnome has advantage on Dexterity (Stealth) checks made to
      hide in rocky terrain.
    Gnome Cunning.
      The gnome has advantage on Intelligence, Wisdom, and Charisma
      saving throws against magic.
    Innate Spellcasting.
      The gnome's innate spellcasting ability is Intelligence (spell
      save DC 11). It can innately cast the following spells,
      requiring no material components:
      
      - At will: nondetection (self only)
      - 1/day each: blindness/deafness, blur, disguise self
      
    War Pick
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 6
      (1d8 + 2) piercing damage.
    Poisoned Dart
      Ranged Weapon Attack: +4 to hit, range 30/120 ft., one
      creature. Hit: 4 (1d4 + 2) piercing damage, and the target must
      succeed on a DC 12 Constitution saving throw or be poisoned for
      1 minute. The target can repeat the saving throw at the end of
      each of its turns, ending the effect on itself on a success
    
    """
    name = "Deep Gnome (Svirfneblin)"
    description = "Small humanoid, neutral good"
    challenge_rating = 0.5
    armor_class = 15
    skills = "Investigation +3, Perception +2, Stealth +4"
    senses = "Darkvision 120 ft., Passive Perception 12"
    languages = "Gnomish, Terran, Undercommon"
    strength = Ability(15)
    dexterity = Ability(14)
    constitution = Ability(14)
    intelligence = Ability(12)
    wisdom = Ability(10)
    charisma = Ability(9)
    speed = 20
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 16
    hit_dice = "3d6"


DeepGnome = DeepGnomeSvirfneblin


class Deer(Monster):
    """
    Bite.
      Melee Weapon Attack: +2 to hit, reach 5 ft., one target. Hit: 2 (1d4)
      piercing damage.
    """
    name = "Deer"
    description = "Medium beast, unaligned"
    challenge_rating = 0
    armor_class = 13
    skills = ""
    senses = "Passive Perception 12"
    languages = ""
    strength = Ability(11)
    dexterity = Ability(16)
    constitution = Ability(11)
    intelligence = Ability(2)
    wisdom = Ability(14)
    charisma = Ability(5)
    speed = 50
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 4
    hit_dice = "1d8"
    spells = []


class Deva(Monster):
    """
    Angelic Weapons.
      The deva's weapon attacks are magical. When the deva hits with any
      weapon, the weapon deals an extra 4d8 radiant damage (included in the
      attack).
    Innate Spellcasting.
      The deva's spellcasting ability is Charisma (spell save DC 17). The
      deva can innately cast the following spells, requiring only verbal
      components:
      
      At will: detect evil and good
      
      1/day each: commune, raise dead
    Magic Resistance.
      The deva has advantage on saving throws against spells and other
      magical effects.
    Multiattack.
      The deva makes two melee attacks.
    Mace.
      Melee Weapon Attack: +8 to hit, reach 5 ft., one target. Hit: 7 (1d6 +
      4) bludgeoning damage plus 18 (4d8) radiant damage.
    Healing Touch.
      The deva touches another creature. The target magically regains 20
      (4d8 + 2) hit points and is freed from any curse, disease, poison,
      blindness, or deafness.
    Change Shape.
      The deva magically polymorphs into a humanoid or beast that has a
      challenge rating equal to or less than its own, or back into its true
      form. It reverts to its true form if it dies. Any equipment it is
      wearing or carrying is absorbed or borne by the new form (the deva's
      choice).
      
      In a new form, the deva retains its game statistics and ability to
      speak, but its AC, movement modes, Strength, Dexterity, and special
      senses are replaced by those of the new form, and it gains any
      statistics and capabilities (except class features, legendary actions,
      and lair actions) that the new form has but that it lacks.
    """
    name = "Deva"
    description = "Medium celestial, lawful good"
    challenge_rating = 10
    armor_class = 17
    skills = "Insight +9, Perception +9"
    senses = "Darkvision 120 ft., Passive Perception 19"
    languages = "all, telepathy 120 ft."
    strength = Ability(18)
    dexterity = Ability(18)
    constitution = Ability(18)
    intelligence = Ability(17)
    wisdom = Ability(20)
    charisma = Ability(20)
    speed = 30
    swim_speed = 0
    fly_speed = 90
    climb_speed = 0
    hp_max = 136
    hit_dice = "16d8"
    spells = []


class DireWolf(Monster):
    """
    Keen Hearing and Smell.
      The wolf has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Pack Tactics.
      The wolf has advantage on an attack roll against a creature if at
      least one of the wolf's allies is within 5 ft. of the creature and the
      ally isn't incapacitated.
    Bite.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 10 (2d6
      + 3) piercing damage. If the target is a creature, it must succeed on
      a DC 13 Strength saving throw or be knocked prone.
    """
    name = "Dire Wolf"
    description = "Large beast, unaligned"
    challenge_rating = 1
    armor_class = 14
    skills = "Perception +3, Stealth +4"
    senses = "Passive Perception 13"
    languages = ""
    strength = Ability(17)
    dexterity = Ability(15)
    constitution = Ability(15)
    intelligence = Ability(3)
    wisdom = Ability(12)
    charisma = Ability(7)
    speed = 50
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 37
    hit_dice = "5d10"
    spells = []


class Djinni(Monster):
    """
    Elemental Demise.
      If the djinni dies, its body disintegrates into a warm breeze, leaving
      behind only equipment the djinni was wearing or carrying.
    Innate Spellcasting.
      The djinni's innate spellcasting ability is Charisma (spell save DC
      17, +9 to hit with spell attacks). It can innately cast the following
      spells, requiring no material components:
      
  
      
      At will: detect evil and good, detect magic, thunderwave
      
      3/day each: create food and water (can create wine instead of water),
      tongues, wind walk
      
      1/day each: conjure elemental (air elemental only), creation, gaseous
      form, invisibility, major image, plane shift
    Variant: Genie Powers.
      Genies have a variety of magical capabilities, including spells. A few
      have even greater powers that allow them to alter their appearance or
      the nature of reality.
      
  
      
      Disguises.
      
      Some genies can veil themselves in illusion to pass as other similarly
      shaped creatures. Such genies can innately cast the disguise self
      spell at will, often with a longer duration than is normal for that
      spell. Mightier genies can cast the true polymorph spell one to three
      times per day, possibly with a longer duration than normal. Such
      genies can change only their own shape, but a rare few can use the
      spell on other creatures and objects as well.
      
      Wishes.
      
      The genie power to grant wishes is legendary among mortals. Only the
      most potent genies, such as those among the nobility, can do so. A
      particular genie that has this power can grant one to three wishes to
      a creature that isn't a genie. Once a genie has granted its limit of
      wishes, it can't grant wishes again for some amount of time (usually 1
      year). and cosmic law dictates that the same genie can expend its
      limit of wishes on a specific creature only once in that creature's
      existence.
      
      To be granted a wish, a creature within 60 feet of the genie states a
      desired effect to it. The genie can then cast the wish spell on the
      creature's behalf to bring about the effect. Depending on the genie's
      nature, the genie might try to pervert the intent of the wish by
      exploiting the wish's poor wording. The perversion of the wording is
      usually crafted to be to the genie's benefit.
    Multiattack.
      The djinni makes three scimitar attacks.
    Scimitar.
      Melee Weapon Attack: +9 to hit, reach 5 ft., one target. Hit: 12 (2d6
      + 5) slashing damage plus 3 (1d6) lightning or thunder damage
      (djinni's choice).
    Create Whirlwind.
      A 5-foot-radius, 30-foot-tall cylinder of swirling air magically forms
      on a point the djinni can see within 120 feet of it. The whirlwind
      lasts as long as the djinni maintains concentration (as if
      concentrating on a spell). Any creature but the djinni that enters the
      whirlwind must succeed on a DC 18 Strength saving throw or be
      restrained by it. The djinni can move the whirlwind up to 60 feet as
      an action, and creatures restrained by the whirlwind move with it. The
      whirlwind ends if the djinni loses sight of it.
      
      A creature can use its action to free a creature restrained by the
      whirlwind, including itself, by succeeding on a DC 18 Strength check.
      If the check succeeds, the creature is no longer restrained and moves
      to the nearest space outside the whirlwind.
    """
    name = "Djinni"
    description = "Large elemental, chaotic good"
    challenge_rating = 11
    armor_class = 17
    skills = ""
    senses = "Darkvision 120 ft., Passive Perception 13"
    languages = "Auran"
    strength = Ability(21)
    dexterity = Ability(15)
    constitution = Ability(22)
    intelligence = Ability(15)
    wisdom = Ability(16)
    charisma = Ability(20)
    speed = 30
    swim_speed = 0
    fly_speed = 90
    climb_speed = 0
    hp_max = 161
    hit_dice = "14d10"
    spells = []


class Doppelganger(Monster):
    """
    Shapechanger.
      The doppelganger can use its action to polymorph into a Small or
      Medium humanoid it has seen, or back into its true form. Its
      statistics, other than its size, are the same in each form. Any
      equipment it is wearing or carrying isn't transformed. It reverts to
      its true form if it dies.
    Ambusher.
      The doppelganger has advantage on attack rolls against any creature it
      has surprised.
    Surprise Attack.
      If the doppelganger surprises a creature and hits it with an attack
      during the first round of combat, the target takes an extra 10 (3d6)
      damage from the attack.
    Multiattack.
      The doppelganger makes two melee attacks.
    Slam.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 7 (1d6 +
      4) bludgeoning damage.
    Read Thoughts.
      The doppelganger magically reads the surface thoughts of one creature
      within 60 ft. of it. The effect can penetrate barriers, but 3 ft. of
      wood or dirt, 2 ft. of stone, 2 inches of metal, or a thin sheet of
      lead blocks it. While the target is in range, the doppelganger can
      continue reading its thoughts, as long as the doppelganger's
      concentration isn't broken (as if concentrating on a spell). While
      reading the target's mind, the doppelganger has advantage on Wisdom
      (Insight) and Charisma (Deception, Intimidation, and Persuasion)
      checks against the target.
    """
    name = "Doppelganger"
    description = "Medium monstrosity, unaligned"
    challenge_rating = 3
    armor_class = 14
    skills = "Deception +6, Insight +3"
    senses = "Darkvision 60 ft., Passive Perception 11"
    languages = "Common"
    strength = Ability(11)
    dexterity = Ability(18)
    constitution = Ability(14)
    intelligence = Ability(11)
    wisdom = Ability(12)
    charisma = Ability(14)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 52
    hit_dice = "8d8"
    spells = []


class DraftHorse(Monster):
    """
    Hooves.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 9 (2d4 +
      4) bludgeoning damage.
    """
    name = "Draft Horse"
    description = "Large beast, unaligned"
    challenge_rating = 0.25
    armor_class = 10
    skills = ""
    senses = "Passive Perception 10"
    languages = ""
    strength = Ability(18)
    dexterity = Ability(10)
    constitution = Ability(12)
    intelligence = Ability(2)
    wisdom = Ability(11)
    charisma = Ability(7)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 19
    hit_dice = "3d10"
    spells = []


class DragonTurtle(Monster):
    """
    Amphibious.
      The dragon turtle can breathe air and water.
    Multiattack.
      The dragon turtle makes three attacks: one with its bite and two with
      its claws. It can make one tail attack in place of its two claw
      attacks.
    Bite.
      Melee Weapon Attack: +13 to hit, reach 15 ft., one target. Hit: 26
      (3d12 + 7) piercing damage.
    Claw.
      Melee Weapon Attack: +13 to hit, reach 10 ft., one target. Hit: 16
      (2d8 + 7) slashing damage.
    Tail.
      Melee Weapon Attack: +13 to hit, reach 15 ft., one target. Hit: 26
      (3d12 + 7) bludgeoning damage. If the target is a creature, it must
      succeed on a DC 20 Strength saving throw or be pushed up to 10 feet
      away from the dragon turtle and knocked prone.
    Steam Breath.
      The dragon turtle exhales scalding steam in a 60-foot cone. Each
      creature in that area must make a DC 18 Constitution saving throw,
      taking 52 (15d6) fire damage on a failed save, or half as much damage
      on a successful one. Being underwater doesn't grant resistance against
      this damage.
    """
    name = "Dragon Turtle"
    description = "Gargantuan dragon, neutral"
    challenge_rating = 17
    armor_class = 20
    skills = ""
    senses = "Darkvision 120 ft., Passive Perception 11"
    languages = "Aquan, Draconic"
    strength = Ability(25)
    dexterity = Ability(10)
    constitution = Ability(20)
    intelligence = Ability(10)
    wisdom = Ability(12)
    charisma = Ability(12)
    speed = 20
    swim_speed = 40
    fly_speed = 0
    climb_speed = 0
    hp_max = 341
    hit_dice = "22d20"
    spells = []


class Dretch(Monster):
    """
    Multiattack.
      The dretch makes two attacks: one with its bite and one with its
      claws.
    Bite.
      Melee Weapon Attack: +2 to hit, reach 5 ft., one target. Hit: 3 (1d6)
      piercing damage.
    Claws.
      Melee Weapon Attack: +2 to hit, reach 5 ft., one target. Hit: 5 (2d4)
      slashing damage.
    Fetid Cloud.
      A 10-foot radius of disgusting green gas extends out from the dretch.
      The gas spreads around corners, and its area is lightly obscured. It
      lasts for 1 minute or until a strong wind disperses it. Any creature
      that starts its turn in that area must succeed on a DC 11 Constitution
      saving throw or be poisoned until the start of its next turn. While
      poisoned in this way, the target can take either an action or a bonus
      action on its turn, not both, and can't take reactions.
    """
    name = "Dretch"
    description = "Small fiend, chaotic evil"
    challenge_rating = 0.25
    armor_class = 11
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 9"
    languages = "Abyssal, telepathy 60 ft. (works only with creatures that understand Abyssal)"
    strength = Ability(11)
    dexterity = Ability(11)
    constitution = Ability(12)
    intelligence = Ability(5)
    wisdom = Ability(8)
    charisma = Ability(3)
    speed = 20
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 18
    hit_dice = "4d6"
    spells = []


class Drider(Monster):
    """
    Fey Ancestry.
      The drider has advantage on saving throws against being charmed, and
      magic can't put the drider to sleep.
    Innate Spellcasting.
      The drider's innate spellcasting ability is Wisdom (spell save DC 13).
      The drider can innately cast the following spells, requiring no
      material components:
      
      At will: dancing lights
      
      1/day each: darkness, faerie fire
    Spider Climb.
      The drider can climb difficult surfaces, including upside down on
      ceilings, without needing to make an ability check.
    Sunlight Sensitivity.
      While in sunlight, the drider has disadvantage on attack rolls, as
      well as on Wisdom (Perception) checks that rely on sight.
    Web Walker.
      The drider ignores movement restrictions caused by webbing.
    Multiattack.
      The drider makes three attacks, either with its longsword or its
      longbow. It can replace one of those attacks with a bite attack.
    Bite.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one creature. Hit: 2
      (1d4) piercing damage plus 9 (2d8) poison damage.
    Longsword.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 7 (1d8 +
      3) slashing damage, or 8 (1d10 + 3) slashing damage if used with two
      hands.
    Longbow.
      Ranged Weapon Attack: +6 to hit, range 150/600 ft., one target. Hit: 7
      (1d8 + 3) piercing damage plus 4 (1d8) poison damage.
    """
    name = "Drider"
    description = "Large monstrosity, chaotic evil"
    challenge_rating = 6
    armor_class = 19
    skills = "Perception +5, Stealth +9"
    senses = "Darkvision 120 ft., Passive Perception 15"
    languages = "Elvish, Undercommon"
    strength = Ability(16)
    dexterity = Ability(16)
    constitution = Ability(18)
    intelligence = Ability(13)
    wisdom = Ability(14)
    charisma = Ability(12)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 30
    hp_max = 123
    hit_dice = "13d10"
    spells = []


class Drow(Monster):
    """
    Fey Ancestry.
      The drow has advantage on saving throws against being charmed, and
      magic can't put the drow to sleep.
    Innate Spellcasting.
      The drow's spellcasting ability is Charisma (spell save DC 11). It can
      innately cast the following spells, requiring no material components:
      
      At will: dancing lights
      
      1/day each: darkness, faerie fire
    Sunlight Sensitivity.
      While in sunlight, the drow has disadvantage on attack rolls, as well
      as on Wisdom (Perception) checks that rely on sight.
    Shortsword.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 5 (1d6 +
      2) piercing damage.
    Hand Crossbow.
      Ranged Weapon Attack: +4 to hit, range 30/120 ft., one target. Hit: 5
      (1d6 + 2) piercing damage, and the target must succeed on a DC 13
      Constitution saving throw or be poisoned for 1 hour. If the saving
      throw fails by 5 or more, the target is also unconscious while
      poisoned in this way. The target wakes up if it takes damage or if
      another creature takes an action to shake it awake.
    """
    name = "Drow"
    description = "Medium humanoid, neutral evil"
    challenge_rating = 0.25
    armor_class = 15
    skills = "Perception +2, Stealth +4"
    senses = "Darkvision 120 ft., Passive Perception 12"
    languages = "Elvish, Undercommon"
    strength = Ability(10)
    dexterity = Ability(14)
    constitution = Ability(10)
    intelligence = Ability(11)
    wisdom = Ability(11)
    charisma = Ability(12)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 13
    hit_dice = "3d8"
    spells = []


class Druid(Monster):
    """
    Spellcasting.
      The druid is a 4th-level spellcaster. Its spellcasting ability is
      Wisdom (spell save DC 12, +4 to hit with spell attacks). It has the
      following druid spells prepared:
      
  
      
      - Cantrips (at will): druidcraft, produce flame, shillelagh
      
      - 1st level (4 slots): entangle, longstrider, speak with animals,
      thunderwave
      
      - 2nd level (3 slots): animal messenger, barkskin
    Quarterstaff.
      Melee Weapon Attack: +2 to hit (+4 to hit with shillelagh), reach 5
      ft., one target. Hit: 3 (1d6) bludgeoning damage, or 6 (1d8 + 2)
      bludgeoning damage with shillelagh or if wielded with two hands.
    """
    name = "Druid"
    description = "Medium humanoid, any alignment"
    challenge_rating = 2
    armor_class = 11
    skills = "Medicine +4, Nature +3, Perception +4"
    senses = "Passive Perception 14"
    languages = "Druidic plus any two languages"
    strength = Ability(10)
    dexterity = Ability(12)
    constitution = Ability(13)
    intelligence = Ability(12)
    wisdom = Ability(15)
    charisma = Ability(11)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 27
    hit_dice = "5d8"
    spells = ["druidcraft", "produce flame", "shillelagh", "entangle", "longstrider", "speak with animals", "thunderwave", "animal messenger", "barkskin"]


class Dryad(Monster):
    """
    Innate Spellcasting.
      The dryad's innate spellcasting ability is Charisma (spell save DC
      14). The dryad can innately cast the following spells, requiring no
      material components:
      
  
      
      At will: druidcraft
      
      3/day each: entangle, goodberry
      
      1/day each: barkskin, pass without trace, shillelagh
    Magic Resistance.
      The dryad has advantage on saving throws against spells and other
      magical effects.
    Speak with Beasts and Plants.
      The dryad can communicate with beasts and plants as if they shared a
      language.
    Tree Stride.
      Once on her turn, the dryad can use 10 ft. of her movement to step
      magically into one living tree within her reach and emerge from a
      second living tree within 60 ft. of the first tree, appearing in an
      unoccupied space within 5 ft. of the second tree. Both trees must be
      large or bigger.
    Club.
      Melee Weapon Attack: +2 to hit (+6 to hit with shillelagh), reach 5
      ft., one target. Hit: 2 (1 d4) bludgeoning damage, or 8 (1d8 + 4)
      bludgeoning damage with shillelagh.
    Fey Charm.
      The dryad targets one humanoid or beast that she can see within 30
      feet of her. If the target can see the dryad, it must succeed on a DC
      14 Wisdom saving throw or be magically charmed. The charmed creature
      regards the dryad as a trusted friend to be heeded and protected.
      Although the target isn't under the dryad's control, it takes the
      dryad's requests or actions in the most favorable way it can.
      
      Each time the dryad or its allies do anything harmful to the target,
      it can repeat the saving throw, ending the effect on itself on a
      success. Otherwise, the effect lasts 24 hours or until the dryad dies,
      is on a different plane of existence from the target, or ends the
      effect as a bonus action. If a target's saving throw is successful,
      the target is immune to the dryad's Fey Charm for the next 24 hours.
      
      The dryad can have no more than one humanoid and up to three beasts
      charmed at a time.
    """
    name = "Dryad"
    description = "Medium fey, neutral"
    challenge_rating = 1
    armor_class = 11
    skills = "Perception +4, Stealth +5"
    senses = "Darkvision 60 ft., Passive Perception 14"
    languages = "Elvish, Sylvan"
    strength = Ability(10)
    dexterity = Ability(12)
    constitution = Ability(11)
    intelligence = Ability(14)
    wisdom = Ability(15)
    charisma = Ability(18)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 22
    hit_dice = "5d8"
    spells = []


class Duergar(Monster):
    """
    Duergar Resilience.
      The duergar has advantage on saving throws against poison, spells, and
      illusions, as well as to resist being charmed or paralyzed.
    Sunlight Sensitivity.
      While in sunlight, the duergar has disadvantage on attack rolls, as
      well as on Wisdom (Perception) checks that rely on sight.
    Enlarge.
      For 1 minute, the duergar magically increases in size, along with
      anything it is wearing or carrying. While enlarged, the duergar is
      Large, doubles its damage dice on Strength-based weapon attacks
      (included in the attacks), and makes Strength checks and Strength
      saving throws with advantage. If the duergar lacks the room to become
      Large, it attains the maximum size possible in the space available.
    War Pick.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 6 (1d8 +
      2) piercing damage, or 11 (2d8 + 2) piercing damage while enlarged.
    Javelin.
      Melee or Ranged Weapon Attack: +4 to hit, reach 5 ft. or range 30/120
      ft., one target. Hit: 5 (1d6 + 2) piercing damage, or 9 (2d6 + 2)
      piercing damage while enlarged.
    Invisibility.
      The duergar magically turns invisible until it attacks, casts a spell,
      or uses its Enlarge, or until its concentration is broken, up to 1
      hour (as if concentrating on a spell). Any equipment the duergar wears
      or carries is invisible with it .
    """
    name = "Duergar"
    description = "Medium humanoid, lawful evil"
    challenge_rating = 1
    armor_class = 16
    skills = ""
    senses = "Darkvision 120 ft., Passive Perception 10"
    languages = "Dwarvish, Undercommon"
    strength = Ability(14)
    dexterity = Ability(11)
    constitution = Ability(14)
    intelligence = Ability(11)
    wisdom = Ability(10)
    charisma = Ability(9)
    speed = 25
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 26
    hit_dice = "4d8"
    spells = []


class DustMephit(Monster):
    """
    Death Burst.
      When the mephit dies, it explodes in a burst of dust. Each creature
      within 5 ft. of it must then succeed on a DC 10 Constitution saving
      throw or be blinded for 1 minute. A blinded creature can repeat the
      saving throw on each of its turns, ending the effect on itself on a
      success.
    Innate Spellcasting.
      The mephit can innately cast sleep, requiring no material components.
      Its innate spellcasting ability is Charisma.
    Claws.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one creature. Hit: 4 (1d4
      + 2) slashing damage.
    Blinding Breath (Recharge 6).
      The mephit exhales a 15-foot cone of blinding dust. Each creature in
      that area must succeed on a DC 10 Dexterity saving throw or be blinded
      for 1 minute. A creature can repeat the saving throw at the end of
      each of its turns, ending the effect on itself on a success.
    Variant: Summon Mephits.
      The mephit has a 25 percent chance of summoning 1d4 mephits of its
      kind. A summoned mephit appears in an unoccupied space within 60 feet
      of its summoner, acts as an ally of its summoner, and can't summon
      other mephits. It remains for 1 minute, until it or its summoner dies,
      or until its summoner dismisses it as an action.
    """
    name = "Dust Mephit"
    description = "Small elemental, neutral evil"
    challenge_rating = 0.5
    armor_class = 12
    skills = "Perception +2, Stealth +4"
    senses = "Darkvision 60 ft., Passive Perception 12"
    languages = "Auran, Terran"
    strength = Ability(5)
    dexterity = Ability(14)
    constitution = Ability(10)
    intelligence = Ability(9)
    wisdom = Ability(11)
    charisma = Ability(10)
    speed = 30
    swim_speed = 0
    fly_speed = 30
    climb_speed = 0
    hp_max = 17
    hit_dice = "5d6"
    spells = []



"""
A collection of monsters from D&D 5e's SRD

This file was autogenerated by https://github.com/stravajiaxen/5e-srd-to-py
"""

from dungeonsheets.monsters.monsters import Monster
from dungeonsheets.stats import Ability


class Jackal(Monster):
    """
    Keen Hearing and Smell.
      The jackal has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Pack Tactics.
      The jackal has advantage on an attack roll against a creature if at
      least one of the jackal's allies is within 5 ft. of the creature and
      the ally isn't incapacitated.
    Bite.
      Melee Weapon Attack: +1 to hit, reach 5 ft., one target. Hit: 1 (1d4 -
      1) piercing damage.
    """
    name = "Jackal"
    description = "Small beast, unaligned"
    challenge_rating = 0
    armor_class = 12
    skills = "Perception +3"
    senses = "Passive Perception 13"
    languages = ""
    strength = Ability(8)
    dexterity = Ability(15)
    constitution = Ability(11)
    intelligence = Ability(3)
    wisdom = Ability(12)
    charisma = Ability(6)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 3
    hit_dice = "1d6"
    spells = []
